jQuery(function($) {
    'use strict';

    var wc_verifone_admin = {
        isCardEnabled: function() {
            return ( $('#woocommerce_verifone_hosted_cart_card_enabled').is( ":checked" ) === true );
        },

        isAbsInstallmentEnabled: function() {
            return ( $('#woocommerce_verifone_hosted_cart_abs_installment_enabled').is( ":checked" ) === true );
        },

        generateId: function() {
            return (performance.now().toString(36)+Math.random().toString(36)).replace(/\./g,"");
        },

        checkValidity: function() {
            var result = true;
            $("input.abs_input").each(function( index ) {
                if (isNaN($( this ).val())) {
                    $(this).css("border", "3px solid red");
                    result = false;
                } else {
                    $(this).css("border", "none");
                }

            });
            return result;
        },

        addInterval: function() {
            var row = document.createElement('tr');
            row.setAttribute('class', 'verifone_abs');
            var rowId = wc_verifone_admin.generateId();
            row.setAttribute('id', 'verifone_' + rowId);

            var prevAmount = 0;
            var prevAmountData = document.createElement('td');
            var prevAmountInput = document.createElement('input');
            if ($('.verifone_abs').length > 0) {
                var prevId = jQuery('.verifone_abs').last()[0].id.split('_')[1];
                prevAmount = $('#verifone_max_amount_' + prevId).val();
                prevAmountInput.setAttribute('disabled', 'disabled');
            }

            prevAmountInput.setAttribute('id', 'verifone_prev_amount_' + rowId);
            prevAmountInput.setAttribute('class', 'abs_input');
            prevAmountInput.setAttribute( 'style', 'border-radius: 4px; border: 1px solid #8c8f94;' );
            prevAmountInput.value = prevAmount;
            prevAmountData.appendChild(prevAmountInput);
            row.appendChild(prevAmountData);

            var maxAmountData = document.createElement('td');
            var maxAmountInput = document.createElement('input');
            maxAmountInput.setAttribute('id', 'verifone_max_amount_' + rowId);
            maxAmountInput.setAttribute('class', 'abs_input');
            maxAmountInput.setAttribute( 'style', 'border-radius: 4px; border: 1px solid #8c8f94;' );
            maxAmountData.appendChild(maxAmountInput);
            row.appendChild(maxAmountData);

            var installmentData = document.createElement('td');
            var installmentInput = document.createElement('input');
            installmentInput.setAttribute('id', 'verifone_installment_' + rowId);
            installmentInput.setAttribute('class', 'abs_input');
            installmentInput.setAttribute( 'style', 'border-radius: 4px; border: 1px solid #8c8f94;' );
            installmentData.appendChild(installmentInput);
            row.appendChild(installmentData);

            var deleteData = document.createElement('td');
            var delBtn = document.createElement('a');
            delBtn.setAttribute('class', 'button verifone_installment_delete');
            delBtn.setAttribute( 'style',  'color: #dc3545; text-decoration: underline; border: 0; background: none' )
            delBtn.innerHTML = 'Remove';
            deleteData.appendChild(delBtn);
            row.appendChild(deleteData);
            $('#verifone_add_btn_container').before(row);

            $(".verifone_installment_delete").unbind('click');
            $(".verifone_installment_delete").click(function() {
                $(this).closest("tr").remove();
                wc_verifone_admin.updateIntervals();
            });
            $('.abs_input').change(function() {
                wc_verifone_admin.checkValidity();
                wc_verifone_admin.updateIntervals();
            });
        },

        updateIntervals: function() {
            var maxAmounts = [];
            var newAbsInstallmentThreshholds = [];
            $('.verifone_abs').each(function(i, tr) {
                var id = tr.id.split('_')[1];
                var minAmount = parseInt( $( '#verifone_prev_amount_' + id ).val().toString().replace( /[^0-9]/g,'' ) );
                var maxAmount = parseInt( $( '#verifone_max_amount_' + id ).val().toString().replace( /[^0-9.]/g,'' ) );
                maxAmount = ( minAmount * 1 > maxAmount * 1 ? minAmount : maxAmount );
                $('#verifone_max_amount_' + id)[0].value = maxAmount;
                var installment = parseInt( $( '#verifone_installment_' + id ).val().toString().replace( /[^0-9]/g,'' ) );
                var newAbsInstallmentThreshhold = {
                    min_amount: minAmount,
                    max_amount: maxAmount,
                    installments: installment,
                };
                if (maxAmount && installment && installment >= 1 ) {
                    if ( !maxAmounts.includes(maxAmount) ) {
                        maxAmounts.push(maxAmount);
                        newAbsInstallmentThreshholds.push(newAbsInstallmentThreshhold);
                        $('#verifone_max_amount_' + id).css("border", "none");
                    } else {
                        $('#verifone_max_amount_' + id).css("border", "3px solid red");
                    }
                }
            });

            // Sort by max amount and remap prev amount -> remap prev amount no longer sets the first value to 0, see AETCOE-4669
            newAbsInstallmentThreshholds.sort((a, b) => parseFloat(a.max_amount) > parseFloat(b.max_amount) ? 1 : -1);
            for (var i = 0, l = newAbsInstallmentThreshholds.length; i < l; i++) {
                newAbsInstallmentThreshholds[i].min_amount = (i === 0) ? newAbsInstallmentThreshholds[i].min_amount : newAbsInstallmentThreshholds[i-1].max_amount;
            }
            $('#woocommerce_verifone_hosted_cart_abs_installment_threshholds').val(JSON.stringify(newAbsInstallmentThreshholds));
        },

        updateVisibleTreeItems: (parentObj, element, isVisible) => {
            let elementsToWatch=null;
            try {
                elementsToWatch =JSON.parse(element.attr('hides'));
            }catch(e){

            }
            if (!Array.isArray(elementsToWatch)) return;

            for (let subElementIndex in elementsToWatch) {
                let elementId='#woocommerce_verifone_hosted_cart_' + elementsToWatch[subElementIndex];

                if(isVisible) {
                    $(elementId).closest('tr').show();
                    let isVisibleSubItem=$(elementId).is(':checked');
                    parentObj.updateVisibleTreeItems(parentObj, $(elementId), isVisibleSubItem);
                }else {
                    $(elementId).closest('tr').hide();
                    parentObj.updateVisibleTreeItems(parentObj, $(elementId), isVisible);
                }

            }
        },

        init: function() {
            if ( $('woocommerce_verifone_hosted_cart_abs_installment_settings') ) {
                $('#woocommerce_verifone_hosted_cart_abs_installment_threshholds').parents('tr').hide();

                if (this.isCardEnabled() && this.isAbsInstallmentEnabled() ) {
                    var absInstallmentThreshholds = $('#woocommerce_verifone_hosted_cart_abs_installment_threshholds').val();

                    var absInstallmentThreshholdsArray = JSON.parse(absInstallmentThreshholds);

                    var absInstallmentWrapper = document.createElement('table');
                    absInstallmentWrapper.setAttribute('id', 'woocommerce_verifone_hosted_cart_abs_installment_wrapper');
                    absInstallmentWrapper.setAttribute('style', 'border-collapse: separate;border-spacing: 30px 5px;');
                    var absInstallmentThreshholdsHeaderRow = document.createElement('tr');
                    var descriptionData1 = document.createElement('td');
                    descriptionData1.innerHTML = '<p><strong>Min order amount (ILS)</strong></p>';
                    var descriptionData2 = document.createElement('td');
                    descriptionData2.innerHTML = '<p><strong>Max order amount (ILS)</strong></p>';
                    var descriptionData3 = document.createElement('td');
                    descriptionData3.innerHTML = '<p><strong>Maximum no of installments</strong></p>';
                    var descriptionData4 = document.createElement('td');
                    descriptionData4.innerHTML = '';
                    absInstallmentThreshholdsHeaderRow.appendChild(descriptionData1);
                    absInstallmentThreshholdsHeaderRow.appendChild(descriptionData2);
                    absInstallmentThreshholdsHeaderRow.appendChild(descriptionData3);
                    absInstallmentThreshholdsHeaderRow.appendChild(descriptionData4);
                    absInstallmentWrapper.appendChild(absInstallmentThreshholdsHeaderRow);
                    $('p.submit').before(absInstallmentWrapper);

                    absInstallmentThreshholdsArray.forEach(function (absInstallmentThreshholdObject, absIntallmentNum) {
                        var row = document.createElement('tr');
                        row.setAttribute('class', 'verifone_abs');
                        var rowId = wc_verifone_admin.generateId();
                        row.setAttribute('id', 'verifone_' + rowId);

                        var prevAmountData = document.createElement('td');
                        var prevAmountInput = document.createElement('input');
                        if ( absIntallmentNum != 0 ) {
                            prevAmountInput.setAttribute('disabled', 'disabled');
                        }
                        prevAmountInput.setAttribute('id', 'verifone_prev_amount_' + rowId);
                        prevAmountInput.setAttribute('class', 'abs_input');
                        prevAmountInput.setAttribute( 'style', 'border-radius: 4px; border: 1px solid #8c8f94;' );
                        prevAmountInput.value = absInstallmentThreshholdObject['min_amount'];
                        prevAmountData.appendChild(prevAmountInput);
                        row.appendChild(prevAmountData);

                        var maxAmountData = document.createElement('td');
                        var maxAmountInput = document.createElement('input');
                        maxAmountInput.setAttribute('id', 'verifone_max_amount_' + rowId);
                        maxAmountInput.setAttribute('class', 'abs_input');
                        maxAmountInput.setAttribute( 'style', 'border-radius: 4px; border: 1px solid #8c8f94;' );
                        maxAmountInput.value = absInstallmentThreshholdObject['max_amount'];
                        maxAmountData.appendChild(maxAmountInput);
                        row.appendChild(maxAmountData);

                        var installmentData = document.createElement('td');
                        var installmentInput = document.createElement('input');
                        installmentInput.setAttribute('id', 'verifone_installment_' + rowId);
                        installmentInput.setAttribute('class', 'abs_input');
                        installmentInput.setAttribute( 'style', 'border-radius: 4px; border: 1px solid #8c8f94;' );
                        installmentInput.value = absInstallmentThreshholdObject['installments'];
                        installmentData.appendChild(installmentInput);
                        row.appendChild(installmentData);

                        var deleteData = document.createElement('td');
                        var delBtn = document.createElement('a');
                        delBtn.setAttribute('class', 'button verifone_installment_delete');
                        delBtn.setAttribute( 'style',  'color: #dc3545; text-decoration: underline; border: 0; background: none' )
                        delBtn.innerHTML = 'Remove';
                        deleteData.appendChild(delBtn);
                        row.appendChild(deleteData);

                        absInstallmentWrapper.appendChild(row);
                    });

                    var addBtn = document.createElement('a');
                    addBtn.setAttribute('id', 'verifone_installment_add');
                    addBtn.setAttribute('class', 'button-secondary');
                    addBtn.setAttribute('style', 'float: right; padding: 0 30px;- margin-right: 3px;' );
                    addBtn.innerHTML = 'Add';
                    var btnContainer = document.createElement( 'tr' );
                    btnContainer.setAttribute('id', 'verifone_add_btn_container');
                    btnContainer.append( document.createElement('td') );
                    btnContainer.append( document.createElement('td') );
                    var lastCell = document.createElement('td');
                    lastCell.append( addBtn );
                    btnContainer.append( lastCell );
                    absInstallmentWrapper.append( btnContainer );
                    $( "#abs_input").unbind( "change" );
                    $( "#verifone_installment_delete").unbind( "click" );
                    $( "#verifone_installment_add").unbind( "click" );

                    $('.abs_input').change(function() {
                        wc_verifone_admin.checkValidity();
                        wc_verifone_admin.updateIntervals();
                    });

                    $(".verifone_installment_delete").click(function() {
                        $(this).closest("tr").remove();
                        wc_verifone_admin.updateIntervals();
                    });

                    $("#verifone_installment_add").click(function() {
                        if (wc_verifone_admin.checkValidity()) {
                            wc_verifone_admin.addInterval();
                        }
                    });
                } else {
                    $('#woocommerce_verifone_hosted_cart_abs_installment_wrapper').remove();
                }
            }

            $("#webhook_copy_button").click( function() {
                var $temp = $( "<input>" );
                $("body").append($temp);
                $temp.val( $( "#webhook_response_url" ).text() ).select();
                document.execCommand( "copy" );
                $temp.remove();
            });

            let parentObj=this
            $("[hides]").on('change',function (){
                parentObj.updateVisibleTreeItems( parentObj, $(this), $(this).is(':checked') );
            });
            $( "[hides]" ).each( function () {
                let elementsToWatch = JSON.parse( this.getAttribute('hides') );
                if( Array.isArray( elementsToWatch ) ) {
                    for( let elementToHide in elementsToWatch ) {
                        let fieldToHide = '#woocommerce_verifone_hosted_cart_' + elementsToWatch[elementToHide];
                        let dataLevel=$(fieldToHide).data('level');
                        dataLevel=(undefined===dataLevel || null===dataLevel)?1:parseInt(dataLevel)
                        let paddingLeft = 40 * dataLevel;

                        $( fieldToHide ).closest( 'tr' ).find( 'th' ).css( { "padding-left": `${paddingLeft}px` } );
                        $( this ).trigger( 'change' );
                    }
                }
            });
        },

        updateABS: function() {
            if ( this.isAbsInstallmentEnabled() && this.isCardEnabled() ) {
                $('#woocommerce_verifone_hosted_cart_abs_installment_wrapper').css("display", "table");
            } else {
                $('#woocommerce_verifone_hosted_cart_abs_installment_wrapper').css("display", "none");
            }
        },
    };
    $('#woocommerce_verifone_hosted_cart_abs_installment_enabled').change(function() {
        wc_verifone_admin.init();
    });
    $( '#woocommerce_verifone_hosted_cart_card_enabled' ).change(function() {
        wc_verifone_admin.updateABS();
    });
    wc_verifone_admin.init();
});
